@@ -24,7 +24,7 @@ run ./configure \
 run sed -i -e "s/curl_LDFLAGS =/curl_LDFLAGS = -all-static/" src/Makefile
 
 run make clean
-run make -j${PROCESSORS}
+run make -j${SYSTEM_CPUS}
 run make install
 
 if [ ${NETDATA_BUILD_WITH_DEBUG} -eq 0 ]
