@@ -103,20 +103,11 @@ struct mountinfo *mountinfo_find_by_filesystem_super_option(struct mountinfo *ro
     return NULL;
 }
 
-
-// free a linked list of mountinfo structures
-void mountinfo_free(struct mountinfo *mi) {
-    if(unlikely(!mi))
-        return;
-
-    if(likely(mi->next))
-        mountinfo_free(mi->next);
-
+static void mountinfo_free(struct mountinfo *mi) {
     freez(mi->root);
     freez(mi->mount_point);
     freez(mi->mount_options);
     freez(mi->persistent_id);
-
 /*
     if(mi->optional_fields_count) {
         int i;
@@ -131,6 +122,16 @@ void mountinfo_free(struct mountinfo *mi) {
     freez(mi);
 }
 
+// free a linked list of mountinfo structures
+void mountinfo_free_all(struct mountinfo *mi) {
+    while(mi) {
+        struct mountinfo *t = mi;
+        mi = mi->next;
+
+        mountinfo_free(t);
+    }
+}
+
 static char *strdupz_decoding_octal(const char *string) {
     char *buffer = strdupz(string);
 
@@ -175,10 +176,10 @@ static inline int is_read_only(const char *s) {
 // read the whole mountinfo into a linked list
 struct mountinfo *mountinfo_read(int do_statvfs) {
     char filename[FILENAME_MAX + 1];
-    snprintfz(filename, FILENAME_MAX, "%s/proc/self/mountinfo", global_host_prefix);
+    snprintfz(filename, FILENAME_MAX, "%s/proc/self/mountinfo", netdata_configured_host_prefix);
     procfile *ff = procfile_open(filename, " \t", PROCFILE_FLAG_DEFAULT);
     if(unlikely(!ff)) {
-        snprintfz(filename, FILENAME_MAX, "%s/proc/1/mountinfo", global_host_prefix);
+        snprintfz(filename, FILENAME_MAX, "%s/proc/1/mountinfo", netdata_configured_host_prefix);
         ff = procfile_open(filename, " \t", PROCFILE_FLAG_DEFAULT);
         if(unlikely(!ff)) return NULL;
     }
@@ -293,7 +294,7 @@ struct mountinfo *mountinfo_read(int do_statvfs) {
 
                     struct mountinfo *mt;
                     for(mt = root; mt; mt = mt->next) {
-                        if(unlikely(mt->st_dev == mi->st_dev && !(mi->flags & MOUNTINFO_NO_STAT))) {
+                        if(unlikely(mt->st_dev == mi->st_dev && !(mt->flags & MOUNTINFO_IS_SAME_DEV))) {
                             if(strlen(mi->mount_point) < strlen(mt->mount_point))
                                 mt->flags |= MOUNTINFO_IS_SAME_DEV;
                             else
@@ -319,7 +320,7 @@ struct mountinfo *mountinfo_read(int do_statvfs) {
         }
 
         // check if it has size
-        if(do_statvfs) {
+        if(do_statvfs && !(mi->flags & MOUNTINFO_IS_DUMMY)) {
             struct statvfs buff_statvfs;
             if(unlikely(statvfs(mi->mount_point, &buff_statvfs) < 0)) {
                 mi->flags |= MOUNTINFO_NO_STAT;
