@@ -1,8 +1,6 @@
 #define NETDATA_RRD_INTERNALS 1
 #include "common.h"
 
-#define RRD_DEFAULT_GAP_INTERPOLATIONS 1
-
 void __rrdset_check_rdlock(RRDSET *st, const char *file, const char *function, const unsigned long line) {
     debug(D_RRD_CALLS, "Checking read lock on chart '%s'", st->id);
 
@@ -83,7 +81,7 @@ static inline RRDSET *rrdset_index_find_name(RRDHOST *host, const char *name, ui
     result = avl_search_lock(&host->rrdset_root_index_name, (avl *) (&(tmp.avlname)));
     if(result) {
         RRDSET *st = rrdset_from_avlname(result);
-        if(strcmp(st->magic, RRDSET_MAGIC))
+        if(strcmp(st->magic, RRDSET_MAGIC) != 0)
             error("Search for RRDSET %s returned an invalid RRDSET %s (name %s)", name, st->id, st->name);
 
         // fprintf(stderr, "FOUND: %s\n", name);
@@ -135,11 +133,13 @@ char *rrdset_strncpyz_name(char *to, const char *from, size_t length) {
     return to;
 }
 
-void rrdset_set_name(RRDSET *st, const char *name) {
+int rrdset_set_name(RRDSET *st, const char *name) {
     if(unlikely(st->name && !strcmp(st->name, name)))
-        return;
+        return 1;
 
-    debug(D_RRD_CALLS, "rrdset_set_name() old: %s, new: %s", st->name, name);
+    RRDHOST *host = st->rrdhost;
+
+    debug(D_RRD_CALLS, "rrdset_set_name() old: '%s', new: '%s'", st->name?st->name:"", name);
 
     char b[CONFIG_MAX_VALUE + 1];
     char n[RRD_ID_LENGTH_MAX + 1];
@@ -147,8 +147,13 @@ void rrdset_set_name(RRDSET *st, const char *name) {
     snprintfz(n, RRD_ID_LENGTH_MAX, "%s.%s", st->type, name);
     rrdset_strncpyz_name(b, n, CONFIG_MAX_VALUE);
 
+    if(rrdset_index_find_name(host, b, 0)) {
+        error("RRDSET: chart name '%s' on host '%s' already exists.", b, host->hostname);
+        return 0;
+    }
+
     if(st->name) {
-        rrdset_index_del_name(st->rrdhost, st);
+        rrdset_index_del_name(host, st);
         st->name = config_set_default(st->config_section, "name", b);
         st->hash_name = simple_hash(st->name);
         rrdsetvar_rename_all(st);
@@ -164,18 +169,22 @@ void rrdset_set_name(RRDSET *st, const char *name) {
         rrddimvar_rename_all(rd);
     rrdset_unlock(st);
 
-    if(unlikely(rrdset_index_add_name(st->rrdhost, st) != st))
+    if(unlikely(rrdset_index_add_name(host, st) != st))
         error("RRDSET: INTERNAL ERROR: attempted to index duplicate chart name '%s'", st->name);
+
+    return 1;
 }
 
 inline void rrdset_is_obsolete(RRDSET *st) {
+    RRDHOST *host = st->rrdhost;
+
     if(unlikely(!(rrdset_flag_check(st, RRDSET_FLAG_OBSOLETE)))) {
         rrdset_flag_set(st, RRDSET_FLAG_OBSOLETE);
         rrdset_flag_clear(st, RRDSET_FLAG_EXPOSED_UPSTREAM);
 
         // the chart will not get more updates (data collection)
         // so, we have to push its definition now
-        if(unlikely(st->rrdhost->rrdpush_enabled))
+        if(unlikely(host->rrdpush_send_enabled))
             rrdset_push_chart_definition(st);
     }
 }
@@ -191,6 +200,7 @@ inline void rrdset_isnot_obsolete(RRDSET *st) {
 }
 
 inline void rrdset_update_heterogeneous_flag(RRDSET *st) {
+    RRDHOST *host = st->rrdhost;
     RRDDIM *rd;
 
     rrdset_flag_clear(st, RRDSET_FLAG_HOMEGENEOUS_CHECK);
@@ -206,7 +216,7 @@ inline void rrdset_update_heterogeneous_flag(RRDSET *st) {
                 info("Dimension '%s' added on chart '%s' of host '%s' is not homogeneous to other dimensions already present (algorithm is '%s' vs '%s', multiplier is " COLLECTED_NUMBER_FORMAT " vs " COLLECTED_NUMBER_FORMAT ", divisor is " COLLECTED_NUMBER_FORMAT " vs " COLLECTED_NUMBER_FORMAT ").",
                         rd->name,
                         st->name,
-                        st->rrdhost->hostname,
+                        host->hostname,
                         rrd_algorithm_name(rd->algorithm), rrd_algorithm_name(algorithm),
                         rd->multiplier, multiplier,
                         rd->divisor, divisor
@@ -287,7 +297,9 @@ static inline void last_updated_time_align(RRDSET *st) {
 void rrdset_free(RRDSET *st) {
     if(unlikely(!st)) return;
 
-    rrdhost_check_wrlock(st->rrdhost);  // make sure we have a write lock on the host
+    RRDHOST *host = st->rrdhost;
+
+    rrdhost_check_wrlock(host);  // make sure we have a write lock on the host
     rrdset_wrlock(st);                  // lock this RRDSET
 
     // info("Removing chart '%s' ('%s')", st->id, st->name);
@@ -295,10 +307,10 @@ void rrdset_free(RRDSET *st) {
     // ------------------------------------------------------------------------
     // remove it from the indexes
 
-    if(unlikely(rrdset_index_del(st->rrdhost, st) != st))
+    if(unlikely(rrdset_index_del(host, st) != st))
         error("RRDSET: INTERNAL ERROR: attempt to remove from index chart '%s', removed a different chart.", st->id);
 
-    rrdset_index_del_name(st->rrdhost, st);
+    rrdset_index_del_name(host, st);
 
     // ------------------------------------------------------------------------
     // free its children structures
@@ -307,22 +319,25 @@ void rrdset_free(RRDSET *st) {
     while(st->alarms)     rrdsetcalc_unlink(st->alarms);
     while(st->dimensions) rrddim_free(st, st->dimensions);
 
-    rrdfamily_free(st->rrdhost, st->rrdfamily);
+    rrdfamily_free(host, st->rrdfamily);
+
+    debug(D_RRD_CALLS, "RRDSET: Cleaning up remaining chart variables for host '%s', chart '%s'", host->hostname, st->id);
+    rrdvar_free_remaining_variables(host, &st->rrdvar_root_index);
 
     // ------------------------------------------------------------------------
     // unlink it from the host
 
-    if(st == st->rrdhost->rrdset_root) {
-        st->rrdhost->rrdset_root = st->next;
+    if(st == host->rrdset_root) {
+        host->rrdset_root = st->next;
     }
     else {
         // find the previous one
         RRDSET *s;
-        for(s = st->rrdhost->rrdset_root; s && s->next != st ; s = s->next) ;
+        for(s = host->rrdset_root; s && s->next != st ; s = s->next) ;
 
         // bypass it
         if(s) s->next = st->next;
-        else error("Request to free RRDSET '%s': cannot find it under host '%s'", st->id, st->rrdhost->hostname);
+        else error("Request to free RRDSET '%s': cannot find it under host '%s'", st->id, host->hostname);
     }
 
     rrdset_unlock(st);
@@ -334,6 +349,8 @@ void rrdset_free(RRDSET *st) {
 
     // free directly allocated members
     freez(st->config_section);
+    freez(st->plugin_name);
+    freez(st->module_name);
 
     switch(st->rrd_memory_mode) {
         case RRD_MEMORY_MODE_SAVE:
@@ -416,6 +433,8 @@ RRDSET *rrdset_create_custom(
         , const char *context
         , const char *title
         , const char *units
+        , const char *plugin
+        , const char *module
         , long priority
         , int update_every
         , RRDSET_TYPE chart_type
@@ -423,12 +442,30 @@ RRDSET *rrdset_create_custom(
         , long history_entries
 ) {
     if(!type || !type[0]) {
-        fatal("Cannot create rrd stats without a type.");
+        fatal("Cannot create rrd stats without a type: id '%s', name '%s', family '%s', context '%s', title '%s', units '%s', plugin '%s', module '%s'."
+              , (id && *id)?id:"<unset>"
+              , (name && *name)?name:"<unset>"
+              , (family && *family)?family:"<unset>"
+              , (context && *context)?context:"<unset>"
+              , (title && *title)?title:"<unset>"
+              , (units && *units)?units:"<unset>"
+              , (plugin && *plugin)?plugin:"<unset>"
+              , (module && *module)?module:"<unset>"
+        );
         return NULL;
     }
 
     if(!id || !id[0]) {
-        fatal("Cannot create rrd stats without an id.");
+        fatal("Cannot create rrd stats without an id: type '%s', name '%s', family '%s', context '%s', title '%s', units '%s', plugin '%s', module '%s'."
+              , type
+              , (name && *name)?name:"<unset>"
+              , (family && *family)?family:"<unset>"
+              , (context && *context)?context:"<unset>"
+              , (title && *title)?title:"<unset>"
+              , (units && *units)?units:"<unset>"
+              , (plugin && *plugin)?plugin:"<unset>"
+              , (module && *module)?module:"<unset>"
+        );
         return NULL;
     }
 
@@ -495,7 +532,7 @@ RRDSET *rrdset_create_custom(
         if(st) {
             memset(&st->avl, 0, sizeof(avl));
             memset(&st->avlname, 0, sizeof(avl));
-            memset(&st->variables_root_index, 0, sizeof(avl_tree_lock));
+            memset(&st->rrdvar_root_index, 0, sizeof(avl_tree_lock));
             memset(&st->dimensions_index, 0, sizeof(avl_tree_lock));
             memset(&st->rrdset_rwlock, 0, sizeof(netdata_rwlock_t));
 
@@ -507,6 +544,8 @@ RRDSET *rrdset_create_custom(
             st->units = NULL;
             st->context = NULL;
             st->cache_dir = NULL;
+            st->plugin_name = NULL;
+            st->module_name = NULL;
             st->dimensions = NULL;
             st->rrdfamily = NULL;
             st->rrdhost = NULL;
@@ -564,6 +603,9 @@ RRDSET *rrdset_create_custom(
         st->rrd_memory_mode = (memory_mode == RRD_MEMORY_MODE_NONE) ? RRD_MEMORY_MODE_NONE : RRD_MEMORY_MODE_ALLOC;
     }
 
+    st->plugin_name = plugin?strdup(plugin):NULL;
+    st->module_name = module?strdup(module):NULL;
+
     st->config_section = strdup(config_section);
     st->rrdhost = host;
     st->memsize = size;
@@ -617,19 +659,22 @@ RRDSET *rrdset_create_custom(
     st->last_collected_time.tv_usec = 0;
     st->counter_done = 0;
 
-    st->gap_when_lost_iterations_above = (int) (
-            config_get_number(st->config_section, "gap when lost iterations above", RRD_DEFAULT_GAP_INTERPOLATIONS) + 2);
+    st->gap_when_lost_iterations_above = (int) (gap_when_lost_iterations_above + 2);
 
     st->last_accessed_time = 0;
     st->upstream_resync_time = 0;
 
     avl_init_lock(&st->dimensions_index, rrddim_compare);
-    avl_init_lock(&st->variables_root_index, rrdvar_compare);
+    avl_init_lock(&st->rrdvar_root_index, rrdvar_compare);
 
     netdata_rwlock_init(&st->rrdset_rwlock);
 
-    if(name && *name) rrdset_set_name(st, name);
-    else rrdset_set_name(st, id);
+    if(name && *name && rrdset_set_name(st, name))
+        // we did set the name
+        ;
+    else
+        // could not use the name, use the id
+        rrdset_set_name(st, id);
 
     st->title = config_get(st->config_section, "title", title);
     json_fix_string(st->title);
@@ -640,11 +685,11 @@ RRDSET *rrdset_create_custom(
     host->rrdset_root = st;
 
     if(host->health_enabled) {
-        rrdsetvar_create(st, "last_collected_t", RRDVAR_TYPE_TIME_T, &st->last_collected_time.tv_sec, 0);
-        rrdsetvar_create(st, "collected_total_raw", RRDVAR_TYPE_TOTAL, &st->last_collected_total, 0);
-        rrdsetvar_create(st, "green", RRDVAR_TYPE_CALCULATED, &st->green, 0);
-        rrdsetvar_create(st, "red", RRDVAR_TYPE_CALCULATED, &st->red, 0);
-        rrdsetvar_create(st, "update_every", RRDVAR_TYPE_INT, &st->update_every, 0);
+        rrdsetvar_create(st, "last_collected_t",    RRDVAR_TYPE_TIME_T,     &st->last_collected_time.tv_sec, RRDVAR_OPTION_DEFAULT);
+        rrdsetvar_create(st, "collected_total_raw", RRDVAR_TYPE_TOTAL,      &st->last_collected_total,       RRDVAR_OPTION_DEFAULT);
+        rrdsetvar_create(st, "green",               RRDVAR_TYPE_CALCULATED, &st->green,                      RRDVAR_OPTION_DEFAULT);
+        rrdsetvar_create(st, "red",                 RRDVAR_TYPE_CALCULATED, &st->red,                        RRDVAR_OPTION_DEFAULT);
+        rrdsetvar_create(st, "update_every",        RRDVAR_TYPE_INT,        &st->update_every,               RRDVAR_OPTION_DEFAULT);
     }
 
     if(unlikely(rrdset_index_add(host, st) != st))
@@ -665,7 +710,7 @@ RRDSET *rrdset_create_custom(
 // RRDSET - data collection iteration control
 
 inline void rrdset_next_usec_unfiltered(RRDSET *st, usec_t microseconds) {
-    if(unlikely(!st->last_collected_time.tv_sec || !microseconds || (st->counter % remote_clock_resync_iterations) == 0)) {
+    if(unlikely(!st->last_collected_time.tv_sec || !microseconds)) {
         // call the full next_usec() function
         rrdset_next_usec(st, microseconds);
         return;
@@ -692,7 +737,7 @@ inline void rrdset_next_usec(RRDSET *st, usec_t microseconds) {
 
         if(unlikely(since_last_usec < 0)) {
             // oops! the database is in the future
-            info("RRD database for chart '%s' on host '%s' is %0.5Lf secs in the future. Adjusting it to current time.", st->id, st->rrdhost->hostname, (long double)-since_last_usec / USEC_PER_SEC);
+            info("RRD database for chart '%s' on host '%s' is %0.5" LONG_DOUBLE_MODIFIER " secs in the future (counter #%zu, update #%zu). Adjusting it to current time.", st->id, st->rrdhost->hostname, (LONG_DOUBLE)-since_last_usec / USEC_PER_SEC, st->counter, st->counter_done);
 
             st->last_collected_time.tv_sec  = now.tv_sec - st->update_every;
             st->last_collected_time.tv_usec = now.tv_usec;
@@ -706,7 +751,7 @@ inline void rrdset_next_usec(RRDSET *st, usec_t microseconds) {
         }
         else if(unlikely((usec_t)since_last_usec > (usec_t)(st->update_every * 10 * USEC_PER_SEC))) {
             // oops! the database is too far behind
-            info("RRD database for chart '%s' on host '%s' is %0.5Lf secs in the past. Adjusting it to current time.", st->id, st->rrdhost->hostname, (long double)since_last_usec / USEC_PER_SEC);
+            info("RRD database for chart '%s' on host '%s' is %0.5" LONG_DOUBLE_MODIFIER " secs in the past (counter #%zu, update #%zu). Adjusting it to current time.", st->id, st->rrdhost->hostname, (LONG_DOUBLE)since_last_usec / USEC_PER_SEC, st->counter, st->counter_done);
 
             microseconds = (usec_t)since_last_usec;
         }
@@ -731,7 +776,7 @@ static inline usec_t rrdset_init_last_collected_time(RRDSET *st) {
     usec_t last_collect_ut = st->last_collected_time.tv_sec * USEC_PER_SEC + st->last_collected_time.tv_usec;
 
     #ifdef NETDATA_INTERNAL_CHECKS
-    rrdset_debug(st, "initialized last collected time to %0.3Lf", (long double)last_collect_ut / USEC_PER_SEC);
+    rrdset_debug(st, "initialized last collected time to %0.3" LONG_DOUBLE_MODIFIER, (LONG_DOUBLE)last_collect_ut / USEC_PER_SEC);
     #endif
 
     return last_collect_ut;
@@ -744,7 +789,7 @@ static inline usec_t rrdset_update_last_collected_time(RRDSET *st) {
     st->last_collected_time.tv_usec = (suseconds_t) (ut % USEC_PER_SEC);
 
     #ifdef NETDATA_INTERNAL_CHECKS
-    rrdset_debug(st, "updated last collected time to %0.3Lf", (long double)last_collect_ut / USEC_PER_SEC);
+    rrdset_debug(st, "updated last collected time to %0.3" LONG_DOUBLE_MODIFIER, (LONG_DOUBLE)last_collect_ut / USEC_PER_SEC);
     #endif
 
     return last_collect_ut;
@@ -763,7 +808,7 @@ static inline usec_t rrdset_init_last_updated_time(RRDSET *st) {
     usec_t last_updated_ut = st->last_updated.tv_sec * USEC_PER_SEC + st->last_updated.tv_usec;
 
     #ifdef NETDATA_INTERNAL_CHECKS
-    rrdset_debug(st, "initialized last updated time to %0.3Lf", (long double)last_updated_ut / USEC_PER_SEC);
+    rrdset_debug(st, "initialized last updated time to %0.3" LONG_DOUBLE_MODIFIER, (LONG_DOUBLE)last_updated_ut / USEC_PER_SEC);
     #endif
 
     return last_updated_ut;
@@ -822,8 +867,8 @@ static inline size_t rrdset_done_interpolate(
 
         #ifdef NETDATA_INTERNAL_CHECKS
         if(iterations < 0) { error("INTERNAL CHECK: %s: iterations calculation wrapped! first_ut = %llu, last_stored_ut = %llu, next_store_ut = %llu, now_collect_ut = %llu", st->name, first_ut, last_stored_ut, next_store_ut, now_collect_ut); }
-        rrdset_debug(st, "last_stored_ut = %0.3Lf (last updated time)", (long double)last_stored_ut/USEC_PER_SEC);
-        rrdset_debug(st, "next_store_ut  = %0.3Lf (next interpolation point)", (long double)next_store_ut/USEC_PER_SEC);
+        rrdset_debug(st, "last_stored_ut = %0.3" LONG_DOUBLE_MODIFIER " (last updated time)", (LONG_DOUBLE)last_stored_ut/USEC_PER_SEC);
+        rrdset_debug(st, "next_store_ut  = %0.3" LONG_DOUBLE_MODIFIER " (next interpolation point)", (LONG_DOUBLE)next_store_ut/USEC_PER_SEC);
         #endif
 
         last_ut = next_store_ut;
@@ -1029,7 +1074,7 @@ void rrdset_done(RRDSET *st) {
     if(unlikely(netdata_exit)) return;
 
     if(unlikely(st->rrd_memory_mode == RRD_MEMORY_MODE_NONE)) {
-        if(unlikely(st->rrdhost->rrdpush_enabled))
+        if(unlikely(st->rrdhost->rrdpush_send_enabled))
             rrdset_done_push_exclusive(st);
 
         return;
@@ -1039,9 +1084,6 @@ void rrdset_done(RRDSET *st) {
 
     RRDDIM *rd;
 
-    int
-            pthreadoldcancelstate;  // store the old cancelable pthread state, to restore it at the end
-
     char
             store_this_entry = 1,   // boolean: 1 = store this entry, 0 = don't store this entry
             first_entry = 0;        // boolean: 1 = this is the first entry seen for this chart, 0 = all other entries
@@ -1053,8 +1095,7 @@ void rrdset_done(RRDSET *st) {
             next_store_ut,          // the timestamp in microseconds, of the next entry to store in the db
             update_every_ut = st->update_every * USEC_PER_SEC; // st->update_every in microseconds
 
-    if(unlikely(pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &pthreadoldcancelstate) != 0))
-        error("Cannot set pthread cancel state to DISABLE.");
+    netdata_thread_disable_cancelability();
 
     // a read lock is OK here
     rrdset_rdlock(st);
@@ -1066,7 +1107,7 @@ void rrdset_done(RRDSET *st) {
 
     // check if the chart has a long time to be updated
     if(unlikely(st->usec_since_last_update > st->entries * update_every_ut)) {
-        info("host '%s', chart %s: took too long to be updated (%0.3Lf secs). Resetting it.", st->rrdhost->hostname, st->name, (long double)st->usec_since_last_update / USEC_PER_SEC);
+        info("host '%s', chart %s: took too long to be updated (counter #%zu, update #%zu, %0.3" LONG_DOUBLE_MODIFIER " secs). Resetting it.", st->rrdhost->hostname, st->name, st->counter, st->counter_done, (LONG_DOUBLE)st->usec_since_last_update / USEC_PER_SEC);
         rrdset_reset(st);
         st->usec_since_last_update = update_every_ut;
         store_this_entry = 0;
@@ -1154,14 +1195,14 @@ void rrdset_done(RRDSET *st) {
     }
     st->counter_done++;
 
-    if(unlikely(st->rrdhost->rrdpush_enabled))
+    if(unlikely(st->rrdhost->rrdpush_send_enabled))
         rrdset_done_push(st);
 
     #ifdef NETDATA_INTERNAL_CHECKS
-    rrdset_debug(st, "last_collect_ut = %0.3Lf (last collection time)", (long double)last_collect_ut/USEC_PER_SEC);
-    rrdset_debug(st, "now_collect_ut  = %0.3Lf (current collection time)", (long double)now_collect_ut/USEC_PER_SEC);
-    rrdset_debug(st, "last_stored_ut  = %0.3Lf (last updated time)", (long double)last_stored_ut/USEC_PER_SEC);
-    rrdset_debug(st, "next_store_ut   = %0.3Lf (next interpolation point)", (long double)next_store_ut/USEC_PER_SEC);
+    rrdset_debug(st, "last_collect_ut = %0.3" LONG_DOUBLE_MODIFIER " (last collection time)", (LONG_DOUBLE)last_collect_ut/USEC_PER_SEC);
+    rrdset_debug(st, "now_collect_ut  = %0.3" LONG_DOUBLE_MODIFIER " (current collection time)", (LONG_DOUBLE)now_collect_ut/USEC_PER_SEC);
+    rrdset_debug(st, "last_stored_ut  = %0.3" LONG_DOUBLE_MODIFIER " (last updated time)", (LONG_DOUBLE)last_stored_ut/USEC_PER_SEC);
+    rrdset_debug(st, "next_store_ut   = %0.3" LONG_DOUBLE_MODIFIER " (next interpolation point)", (LONG_DOUBLE)next_store_ut/USEC_PER_SEC);
     #endif
 
     // calculate totals and count the dimensions
@@ -1499,6 +1540,5 @@ void rrdset_done(RRDSET *st) {
 
     rrdset_unlock(st);
 
-    if(unlikely(pthread_setcancelstate(pthreadoldcancelstate, NULL) != 0))
-        error("Cannot set pthread cancel state to RESTORE (%d).", pthreadoldcancelstate);
+    netdata_thread_enable_cancelability();
 }
