@@ -142,7 +142,7 @@ int do_proc_sys_devices_system_edac_mc(int update_every, usec_t dt) {
                     , "errors"
                     , "proc"
                     , "/sys/devices/system/edac/mc"
-                    , 6600
+                    , NETDATA_CHART_PRIO_MEM_HW + 50
                     , update_every
                     , RRDSET_TYPE_LINE
             );
@@ -180,7 +180,7 @@ int do_proc_sys_devices_system_edac_mc(int update_every, usec_t dt) {
                     , "errors"
                     , "proc"
                     , "/sys/devices/system/edac/mc"
-                    , 6610
+                    , NETDATA_CHART_PRIO_MEM_HW + 60
                     , update_every
                     , RRDSET_TYPE_LINE
             );
